@@ -10,7 +10,7 @@ use warnings;
 use base qw( IO::Async::Listener );
 IO::Async::Listener->VERSION( '0.61' );
 
-our $VERSION = '0.08';
+our $VERSION = '0.09';
 
 use Carp;
 
@@ -48,8 +48,7 @@ C<Net::Async::HTTP::Server> - serve HTTP with C<IO::Async>
 
  $httpserver->listen(
     addr => { family => "inet6", socktype => "stream", port => 8080 },
-    on_listen_error => sub { die "Cannot listen - $_[-1]\n" },
- );
+ )->get
 
  $loop->run;
 
@@ -103,7 +102,8 @@ sub _init
    my ( $params ) = @_;
 
    $params->{handle_class} = "Net::Async::HTTP::Server::Protocol";
-   $params->{request_class} = "Net::Async::HTTP::Server::Request";
+
+   $params->{request_class} ||= "Net::Async::HTTP::Server::Request";
 
    $self->SUPER::_init( $params );
 }
@@ -148,7 +148,9 @@ sub on_accept
    return $conn;
 }
 
-=head2 $request = $server->make_request( @args )
+=head2 make_request
+
+   $request = $server->make_request( @args )
 
 Invoked by the protocol stream handler to create a new request object
 representing an incoming request. This is provided as a method for subclasses
