@@ -12,7 +12,7 @@ use Carp;
 
 use base qw( Net::Async::HTTP::Server );
 
-our $VERSION = '0.08';
+our $VERSION = '0.09';
 
 use HTTP::Response;
 
@@ -45,8 +45,7 @@ C<Net::Async::HTTP::Server::PSGI> - use C<PSGI> applications with C<Net::Async::
 
  $httpserver->listen(
     addr => { family => "inet6", socktype => "stream", port => 8080 },
-    on_listen_error => sub { die "Cannot listen - $_[-1]\n" },
- );
+ )->get;
 
  $loop->run;
 
