@@ -9,7 +9,7 @@ use strict;
 use warnings;
 use base qw( IO::Async::Stream );
 
-our $VERSION = '0.08';
+our $VERSION = '0.09';
 
 use Carp;
 use Scalar::Util qw( weaken );
@@ -29,6 +29,11 @@ sub on_read
    my $header = $1;
 
    my $request = HTTP::Request->parse( $header );
+   unless( $request and defined $request->protocol and $request->protocol =~ m/^HTTP/ ) {
+      $self->close_now;
+      return 0;
+   }
+
    my $request_body_len = $request->content_length || 0;
 
    $self->debug_printf( "REQUEST %s %s", $request->method, $request->uri->path );
