@@ -8,10 +8,13 @@ package Net::Async::HTTP::Server::Request;
 use strict;
 use warnings;
 
-our $VERSION = '0.08';
+our $VERSION = '0.09';
 
 use Carp;
 
+use URI;
+use URI::QueryParam;
+
 my $CRLF = "\x0d\x0a";
 
 =head1 NAME
@@ -45,7 +48,9 @@ sub new
 
 =cut
 
-=head2 $is_closed = $request->is_closed
+=head2 is_closed
+
+   $is_closed = $request->is_closed
 
 Returns true if the underlying network connection for this request has already
 been closed. If this is the case, the application is free to drop the request
@@ -65,7 +70,9 @@ sub is_closed
    return $self->{is_closed};
 }
 
-=head2 $method = $request->method
+=head2 method
+
+   $method = $request->method
 
 Return the method name from the request header.
 
@@ -77,7 +84,9 @@ sub method
    return $self->{req}->method;
 }
 
-=head2 $path = $request->path
+=head2 path
+
+   $path = $request->path
 
 Return the path name from the request header.
 
@@ -89,7 +98,9 @@ sub path
    return $self->{req}->uri->path;
 }
 
-=head2 $query_string = $request->query_string
+=head2 query_string
+
+   $query_string = $request->query_string
 
 Return the query string from the request header.
 
@@ -101,7 +112,61 @@ sub query_string
    return $self->{req}->uri->query;
 }
 
-=head2 $protocol = $request->protocol
+=head2 query_form
+
+   %params = $request->query_form
+
+I<Since version 0.09.>
+
+Return an even-sized list of name and value pairs that gives the decoded data
+in the query string. This is the same format as the same-named method on
+L<URI>.
+
+=cut
+
+sub query_form
+{
+   my $self = shift;
+   return $self->{req}->uri->query_form;
+}
+
+=head2 query_param_names
+
+   @names = $request->query_param_names
+
+I<Since version 0.09.>
+
+Return a list of the names of all the query parameters.
+
+=cut
+
+sub query_param_names
+{
+   my $self = shift;
+   return $self->{req}->uri->query_param;
+}
+
+=head2 query_param
+
+   $value = $request->query_param( $name )
+
+   @values = $request->query_param( $name )
+
+I<Since version 0.09.>
+
+Return the value or values of a single decoded query parameter.
+
+=cut
+
+sub query_param
+{
+   my $self = shift;
+   return $self->{req}->uri->query_param( @_ );
+}
+
+=head2 protocol
+
+   $protocol = $request->protocol
 
 Return the protocol version from the request header. This will be the full
 string, such as C<HTTP/1.1>.
@@ -114,7 +179,9 @@ sub protocol
    return $self->{req}->protocol;
 }
 
-=head2 $value = $request->header( $key )
+=head2 header
+
+   $value = $request->header( $key )
 
 Return the value of a request header.
 
@@ -127,7 +194,9 @@ sub header
    return $self->{req}->header( $key );
 }
 
-=head2 @headers = $request->headers
+=head2 headers
+
+   @headers = $request->headers
 
 Returns a list of 2-element C<ARRAY> refs containing all the request headers.
 Each referenced array contains, in order, the name and the value.
@@ -147,7 +216,9 @@ sub headers
    return @headers;
 }
 
-=head2 $body = $request->body
+=head2 body
+
+   $body = $request->body
 
 Return the body content from the request as a string of bytes.
 
@@ -165,7 +236,7 @@ sub _write_to_stream
    my $self = shift;
    my ( $stream ) = @_;
 
-   while( my $next = shift @{ $self->{pending} } ) {
+   while( defined( my $next = shift @{ $self->{pending} } ) ) {
       $stream->write( $next,
          $self->protocol eq "HTTP/1.0" ?
             ( on_flush => sub { $stream->close } ) :
@@ -176,7 +247,9 @@ sub _write_to_stream
    return $self->{is_done};
 }
 
-=head2 $request->write( $data )
+=head2 write
+
+   $request->write( $data )
 
 Append more data to the response to be written to the client. C<$data> can
 either be a plain string, or a C<CODE> reference to be used in the underlying
@@ -197,7 +270,9 @@ sub write
    $self->{conn}->_flush_requests;
 }
 
-=head2 $request->write_chunk( $data )
+=head2 write_chunk
+
+   $request->write_chunk( $data )
 
 Append more data to the response in the form of an HTTP chunked-transfer
 chunk. This convenience is a shortcut wrapper for prepending the chunk header.
@@ -215,7 +290,9 @@ sub write_chunk
    $self->write( sprintf "%X$CRLF%s$CRLF", $len, $data );
 }
 
-=head2 $request->done
+=head2 done
+
+   $request->done
 
 Marks this response as completed.
 
@@ -233,7 +310,9 @@ sub done
    $self->{conn}->_flush_requests;
 }
 
-=head2 $request->write_chunk_eof
+=head2 write_chunk_eof
+
+   $request->write_chunk_eof
 
 Sends the final EOF chunk and marks this response as completed.
 
@@ -249,7 +328,9 @@ sub write_chunk_eof
    $self->done;
 }
 
-=head2 $req = $request->as_http_request
+=head2 as_http_request
+
+   $req = $request->as_http_request
 
 Returns the data of the request as an L<HTTP::Request> object.
 
@@ -261,7 +342,9 @@ sub as_http_request
    return $self->{req};
 }
 
-=head2 $request->respond( $response )
+=head2 respond
+
+   $request->respond( $response )
 
 Respond to the request using the given L<HTTP::Response> object.
 
@@ -279,7 +362,9 @@ sub respond
    $self->done;
 }
 
-=head2 $request->respond_chunk_header( $response )
+=head2 respond_chunk_header
+
+   $request->respond_chunk_header( $response )
 
 Respond to the request using the given L<HTTP::Response> object to send in
 HTTP/1.1 chunked encoding mode.
@@ -315,7 +400,9 @@ sub respond_chunk_header
    $self->write_chunk( $response->content ) if length $response->content;
 }
 
-=head2 $stream = $request->stream
+=head2 stream
+
+   $stream = $request->stream
 
 Returns the L<IO::Async::Stream> object representing this connection. Usually
 this would be used for such things as inspecting the client's connection
